#ifndef __port_h
#define __port_h

typedef enum {
	Normal,
	Smiley,
#ifdef HUBSYM
	Symbol,
	Cyrillic,
#endif
	DummyMode
} Mode;

class Port {
	friend class Tile;

public:
	Port( char *disp_name, char *color );
	~Port();

	static void close_all()		{ while(first)	delete first; }

	int empty_queue();					// empty event queue of port
	static int wait_event();			// wait for event on any port

	void add_foreign_color( Port *first, char *color_name );
	void add_color( int id, char *color );
	void sub_color( int id );

	void redraw( int x, int y );							// redraw tile (x,y)
	void redraw_score( int x );							// redraw score of player
	void redraw( int x1, int y1, int x2, int y2 );	// redraw X-region
	void draw_block( int p, int x, int y );			// draw score-block

	static void tile_redraw( int fid );					// redraw tile on index fid
	static void flush_all();

	void selected( int x, int y );	// select tile if possible
	unsigned long deselect( );			// deselect/flash/remove tiles
	int remove();							// remove tile when possible

	Display	*display;
	Screen	*screen;
	Window	window;

	GC			gc_tile;
	GC			gc_color_n( int id );

	const char *find_font( const char *pattern, int size );

	void resize( int w, int h );		// recalculate sizes (redraw tiles)

	char *color_name;						// color
	int	 my_id;							// id

	int	selections;						// number of selections
	int	points;							// own points
	static int Points( int id );		// Points of port ...
	static void statistic();

private:
	class Tile	**tile;					// field of normal tiles
	class Tile	*blank;					// closed field
	class Tile	*empty;					// empty field

	class Color	*colors;					// anchor of color list

	static Port *first;					// anchor of port list
	Port	*next;							// next port in list

	int wbsizex, wbsizey;				// world score block size
	int wborder;                     // world border size
	int WWidth()	{ return wsizex*dwidth+wbsizex*nplayers+3*wborder; }
	int WHeight()	{ return wsizey*dheight+2*wborder; }

	int sizex, sizey;						// tile size
	int bsizex, bsizey;					// size of score-blocks
	int border;								// size of border-frame
	int MWidth()	{ return sizex*dwidth+nplayers*bsizex+3*border; }
	int MHeight()	{ return sizey*dheight+2*border; }
	void MInit(int x) {
		sizex  = x;
		sizey  = x*wsizey /wsizex;
		bsizex = x*wbsizex/wsizex;
		bsizey = x*wbsizey/wsizex;
		border = x*wborder/wsizex;
	}

	int offx, offy;						// window offset for justification
	int width,height;						// size of window (MWidth/MHeight)
	Mode mode;								// tile style

	int		lock_count;					// number of locked tiles 0,1,2
	int		lock[2];						// fid of locked tiles
	unsigned long lock_time;			// lock until ...
	unsigned long flash_time;			// time of next flash

	static struct fd_set	readfds;
	static int				nfds;

	Atom	WmProtocolsPropId, WmDeleteWindowPropId;

public:
	static int nplayers;					// number of ports
	static int	color_id;				// *current* color id
	static int	dsize;
	static int	dwidth, dheight;		// number of tiles
	static int  wsizex, wsizey;		// world tile size
	static Mode def_mode;				// preselected tile style

	void UpdateTime( const char *time_str );
	static int  def_size;				// preselected tile style

friend class ScalableFont;
};

#endif
