
#ifndef __global_h
#	include "global.H"
#endif
#ifndef __tile_h
#	include "tile.H"
#endif
#ifndef __port_h
#	include "port.H"
#endif
#ifndef __font_h
#	include "font.H"
#endif

#ifndef __smiletab_H
#	include "smiletab.H"
#endif

#ifdef HUBSYM
/*
 * to use special ContronicS cyrillic font of Hartmann&Braun
 */
static unsigned char cyr_lock_flag[] = {
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 00 - 0f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 10 - 1f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 20 - 2f */
	0,0,0,0,  0,0,0,0,  0,0,1,1,  1,1,1,1,		/* 30 - 3f */

	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 40 - 4f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 50 - 5f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 60 - 6f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 70 - 7f */

	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 80 - 8f */
	1,1,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* 90 - 9f */
	1,0,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1,		/* a0 - af */
	0,0,0,0,  0,0,0,0,  0,0,0,0,  0,0,0,0,		/* b0 - bf */

	0,0,0,0,  0,0,0,0,  0,0,0,0,  0,0,0,0,		/* c0 - cf */
	0,0,0,0,  0,0,0,0,  0,0,0,0,  0,0,0,0,		/* d0 - df */
	0,0,0,0,  0,0,0,0,  0,0,0,0,  0,0,0,0,		/* e0 - ef */
	1,0,1,1,  1,1,1,1,  1,1,1,1,  1,1,1,1		/* f0 - ff */
};

/*
 * to use special ContronicS symbol font of Hartmann&Braun
 */
static unsigned char symb_lock_flag[] = {
   1,
	0, 0, 0, 0, 0,  0, 1, 0, 0, 1, 	/*   1 -  10 */
	0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  11 -  20 */
	0, 0, 0, 0, 0,  0, 0, 0, 1, 1, 	/*  21 -  30 */
	0, 1, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  31 -  40 */
	0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  41 -  50 */
	1, 0, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  51 -  60 */
	0, 0, 0, 0, 0,  0, 0, 0, 1, 0, 	/*  61 -  70 */
	0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  71 -  80 */
	0, 1, 1, 0, 0,  1, 0, 0, 0, 0, 	/*  81 -  90 */
	0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 	/*  91 - 100 */
	0, 0, 0, 0, 0,  1, 1, 1, 1, 1, 	/* 101 - 110 */
	1, 1, 1, 1, 1,  1, 1, 0, 0, 0, 	/* 111 - 120 */
	0, 0, 0, 0, 0,  1, 1, 1, 1, 1, 	/* 121 - 130 */
   1, 0, 0, 0, 0,  0, 0, 0, 0 };		/* 131 - 139 */
#endif

int Tile::used_syms[128];

void Tile::init_used_syms(int n) {
int i,j;

	switch( Port::def_mode ) {
#ifdef HUBSYM
	case Cyrillic:
		for (i=0;i<sizeof(cyr_lock_flag);i++)		cyr_lock_flag[i]&=1;
		for (i=0;i<n;i++) {
			do {
				j=rand()%sizeof(cyr_lock_flag);
			}
			while(cyr_lock_flag[j]);

			cyr_lock_flag[j] |= 0x02;
			used_syms[i] = j;
		}
		break;
	case Symbol:
		for (i=0;i<sizeof(symb_lock_flag);i++)		symb_lock_flag[i]&=1;
		for (i=0;i<n;i++) {
			do {
				j=rand()%sizeof(symb_lock_flag);
			}
			while(symb_lock_flag[j]);

			symb_lock_flag[j] |= 0x02;
			used_syms[i] = j;
		}
		break;
#endif
	default:
		for (i=0;i<((n<7)?n:7);i++)
			used_syms[i]=i;		// always used the standard smileys

		for (   ;i<n;i++) {
			used_syms[i]=rand()%(sizeof(smiletab)/sizeof(char*));
			for (j=0;j<i;j++) {
				if (used_syms[i]==used_syms[j]) {
					i--;
					break;
				}
			}
		}
		break;
	}
}


Tile::Tile( Port *p, ScalableFont *font, int id )
{
char buffer[40];

#ifdef HUBSYM
	if (p->mode==Cyrillic||p->mode==Symbol) {
		buffer[0] = (id>=0)?used_syms[id]:32;
		buffer[1] = 0;
	}
	else
#endif
	if (p->mode==Smiley) {
		if (id>=0)	strcpy(buffer,smiletab[used_syms[id]]);
		else			strcpy(buffer,"");
	}
	else {
// check justification of sign
		if ((id>=0)&&(id<=9))	buffer[0] = id+48;
		if ((id>=10)&&(id<=35))	buffer[0] = id-10+65;
		if ((id>=36)&&(id<=61))	buffer[0] = id-36+97;
		if (id>61)					buffer[0] = id-62+60;
		if (id>67)					buffer[0] = id-67+33;
		buffer[1] = 0;
	}

// create Pixmap
	display = p->display;	// for automatic destruction

	pixmap = XCreatePixmap( display, RootWindowOfScreen(p->screen),
							p->sizex, p->sizey, DefaultDepthOfScreen(p->screen) );

// Background
	XSetForeground( display, p->gc_tile, (id!=-1)?bg:fg );
	XFillRectangle( display, pixmap, p->gc_tile, 0, 0, p->sizex, p->sizey );

// TopLeft highlight
	XSetForeground( display, p->gc_tile, (id>=-1)?dark:light);
	XDrawLine( display, pixmap, p->gc_tile, 0, 0, p->sizex-1, 0 );
	XDrawLine( display, pixmap, p->gc_tile, 0, 0, 0, p->sizey-1 );
#ifdef DoubleDepth
	if (id==-2) {
		XDrawLine( display, pixmap, p->gc_tile, 1, 1, p->sizex-2, 1 );
		XDrawLine( display, pixmap, p->gc_tile, 1, 1, 1, p->sizey-2 );
	}
#endif
	if (id>=0) {
#ifdef DoubleString
		font->DrawCentered(pixmap,p->gc_tile,
								p->sizex/2+2,p->sizey/2+2, buffer);
#endif
		font->DrawCentered(pixmap,p->gc_tile,
								p->sizex/2+1,p->sizey/2+1, buffer);
	}

// BottomRight shade
	XSetForeground( display, p->gc_tile, (id>=-1)?light:dark);
	XDrawLine( display, pixmap, p->gc_tile, 0, p->sizey-1, p->sizex-1, p->sizey-1);
	XDrawLine( display, pixmap, p->gc_tile, p->sizex-1, 0, p->sizex-1, p->sizey-1);
#ifdef DoubleDepth
	if (id==-2) {
	 XDrawLine( display, pixmap, p->gc_tile, 1, p->sizey-2, p->sizex-2, p->sizey-2);
	 XDrawLine( display, pixmap, p->gc_tile, p->sizex-2, 1, p->sizex-2, p->sizey-2);
	}
#endif
	if (id>=0) {
#ifdef DoubleString
		font->DrawCentered(pixmap,p->gc_tile,
								p->sizex/2-2,p->sizey/2-2, buffer);
#endif
		font->DrawCentered(pixmap,p->gc_tile,
								p->sizex/2-1,p->sizey/2-1, buffer);
	}

// Foreground
	XSetForeground( display, p->gc_tile, fg );
	if (id>=0)
		font->DrawCentered(pixmap,p->gc_tile,
								p->sizex/2,p->sizey/2, buffer);
}


Tile::~Tile() {
	XFreePixmap( display, pixmap );
}

