/*
 * Memory mit Scaleable Fonts und mehreren geoeffneten Bildschirmen
 */

#ifndef __global_h
#	include "global.H"
#endif
#ifndef __port_h
#	include "port.H"
#endif
#ifndef __board_h
#	include "board.H"
#endif
#ifndef __game_h
#	include "game.H"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
// #include <sys/time.h>


#define	_DoubleString
#define	DoubleDepth

#define	_TRACE_COLORS

Board *board_p;

#ifndef FONT_DEFAULT
#	define	FONT_DEFAULT "-bitstream-charter-bold-r-*"
#endif

Parameter param[] = {

// parameters for the 'letter' game

{ FONT_DEFAULT, 0, {
	{  4,  5, 44, 100, 100 }, 		// small
	{  8,  9, 44, 100, 100 }, 		// normal
	{ 11, 12, 44, 100, 100 }		// big
}},

// parameters for the 'smiley' game

{ FONT_DEFAULT, 1, {
	{  6,  3, 44, 100, 270 }, 		// small
	{  8,  3, 44, 100, 270 }, 		// normal
	{ 10,  4, 44, 100, 270 }		// big
}}

#ifdef HUBSYM
, // to append to last entry

// parameter for a symbol game (requires special symbol font)

{ "-hub-symbol-normal-r-normal-*", 0, {
	{  4,  5, 44, 100, 70 }, 
	{  6,  9, 44, 100, 70 }, 
	{  8, 11, 44, 100, 70 }
}},

// parameters for the 'cyrillic' game

{ "-hub-textbook-bold-r-normal-*", 0, {
	{  4,  5, 44, 100, 100 }, 		// small
	{  8,  9, 44, 100, 100 }, 		// normal
	{ 11, 12, 44, 100, 100 }		// big
}}
#endif
};


//////////////////////////////////////////////////////////////////////////////


void usage() {
#ifdef HUBSYM
	printf("\nusage: xmemory [-letter|-symbol|-cyrillic] [small|big] [display/color]*\n" );
#else
	printf("\nusage: xmemory [-letter] [-small|-big] [display/color]*\n" );
#endif
	printf("\nAuthor:  Helmut Hoenig, October-3-95 (V2.0)   (Helmut.Hoenig@hub.de)\n" );
	printf("\ne.g.:  xmemory / barclay:0.0/ genesis:0.0/green\n" );
	printf("       to play with 3 players at:\n" );
	printf("       - the local display with default color\n" );
	printf("       - the barclay:0.0 with default color\n" );
	printf("       - the genesis:0.0 with green color\n" );
	printf("\nsmail (for gifts :-) \n");
	printf("        Helmut Hoenig\n");
	printf("        Hopfenstrasse 8a\n");
	printf("        65520 Bad Camberg\n");
	printf("        GERMANY\n");
	exit(-1);
}

char	*def_col[] = {		// predefined colors for the first players
	"red",
	"DodgerBlue",
	"OliveDrab",
	"gold",
	"DeepPink",
	"tomato",
	"aquamarine",
	"brown",
	"bisque",
	"sienna" };

main( int argc, char **argv )
{
int i;
long	timefield;
char	*color_name;
char	main_display[30];
char	*main_p;

	main_p = getenv("DISPLAY");
	if (main_p) {
		strcpy(main_display,main_p);
		main_p=strstr(main_display,"_XT");
		if (main_p)		main_p+=3;
	}

	time(&timefield);					// preset random generator
	srand((int)timefield);

	Port::nplayers = (argc-1);
	Port::def_size = 1;

	if (argc<2)	usage();
	for (i=1;i<argc;i++) {
		if (!strcmp(argv[i],"-small")) {
			Port::def_size=0;
		}
		else if (!strcmp(argv[i],"-normal")) {
			Port::def_size=1;
		}
		else if (!strcmp(argv[i],"-big")) {
			Port::def_size=2;
		}
		else if (!strcmp(argv[i],"-smiley")) {
			Port::def_mode=Smiley;
		}
		else if (!strcmp(argv[i],"-font")) {
			param[Normal].font = argv[++i];
			param[Smiley].font = argv[  i];
		}
		else if (!strcmp(argv[i],"-letter")) {
			Port::def_mode=Normal;
		}
#ifdef HUBSYM
		else if (!strcmp(argv[i],"-cyrillic")) {
			Port::def_mode=Cyrillic;
		}
		else if (!strcmp(argv[i],"-symbol")) {
			Port::def_mode=Symbol;
		}
#endif
		else if (argv[i][0]=='-')	usage();
		else break;
		Port::nplayers--;
	}

	if (!Port::nplayers)	usage();

	Port::dwidth  = param[Port::def_mode].dim[Port::def_size].dwidth;
	Port::dheight = param[Port::def_mode].dim[Port::def_size].dheight;
	Port::dsize   = param[Port::def_mode].dim[Port::def_size].dsize;
	Port::wsizex  = param[Port::def_mode].dim[Port::def_size].wsizex;
	Port::wsizey  = param[Port::def_mode].dim[Port::def_size].wsizey;
	board_p = new Board( Port::dwidth, Port::dheight );

	for (;i<argc;i++) {
		if ( color_name=strchr(argv[i],'/') ) {
			*color_name++ = '\0';
			if (argv[i][0]=='\0'&&main_p) {
				*main_p = '0'+Port::color_id;
				new Port( main_display, *color_name?color_name:def_col[Port::color_id-1] );
			}
			else {
				new Port( argv[i], *color_name?color_name:def_col[Port::color_id-1] );
			}
		}
		else {
			new Port( argv[i], def_col[Port::color_id-1] );
		}
	}
	board_p->reset();

#if (1)
	while(!Port::wait_event());
#else
	int	n=10;
	while(--n)							Port::wait_event();
#endif

	Port::statistic();
	Port::close_all();
	delete board_p;

	/* sleep(5); */
}
